@@ -11,6 +11,7 @@ extern int disable_colors;
 #define REVERSE_COLOR (hascolors?COLOR_PAIR(7):A_REVERSE)
 #define INPUT_COLOR (hascolors?COLOR_PAIR(2):0)
 #define Q_SELECTED_COLOR (hascolors?SELECTED_COLOR: 0)
-#define Q_UNSELECTED_COLOR (hascolors?A_REVERSE: A_REVERSE)
+/*#define Q_UNSELECTED_COLOR (hascolors?A_REVERSE: A_REVERSE)*/
+#define Q_UNSELECTED_COLOR REVERSE_COLOR
 
 extern int sel_mark_color [4];
