@@ -28,13 +28,44 @@
 #include "mouse.h"
 #include "main.h"
 
-static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/dialog.c,v 1.7 1994/08/10 21:17:54 miguel Exp $";
+static char rcsid [] = "$Id: dialog.c,v 1.11 1994/10/13 04:27:26 miguel Exp $";
 static char buffer [4096];
 
 static Dialog *current_dialog = 0;
 WINDOW *top_window;
 
-void (*refresh_fn)();
+static Refresh *refresh_list = 0;
+
+void push_refresh (void (*new_refresh)())
+{
+    Refresh *new;
+
+    new = xmalloc (sizeof (Refresh), "push_refresh");
+    new->next = (struct Refresh *) refresh_list;
+    new->refresh_fn = new_refresh;
+    refresh_list = new;
+}
+
+void pop_refresh ()
+{
+    Refresh *old;
+    
+    if (!refresh_list)
+	fprintf (stderr, "\n\n\nrefresh stack underflow!\n\n\n");
+    else {
+	old = refresh_list;
+	refresh_list = refresh_list->next;
+	free (old);
+    }
+}
+
+void do_refresh ()
+{
+    if (!refresh_list)
+	fprintf (stderr, "\n\nNo refresh handlers defined!\n\n");
+    else 
+	(*(refresh_list->refresh_fn))();
+}
 
 /* Poor man's window puts, it doesn't handle auto-wrap */
 void my_wputs (WINDOW *w, int y, int x, char *text)
@@ -81,10 +112,10 @@ void create_dialog (int cols, int rows, char *header, char *text, int error)
 	wattron (newd->f, ERROR_COLOR | A_BOLD);
 	wattron (newd->t, ERROR_COLOR | A_BOLD);
     } else {
-	wattron (newd->d, A_REVERSE);
+	wattron (newd->d, REVERSE_COLOR);
 	wclr (newd->d);
-	wattron (newd->f, A_REVERSE);
-	wattron (newd->t, A_REVERSE);
+	wattron (newd->f, REVERSE_COLOR);
+	wattron (newd->t, REVERSE_COLOR);
     }
     box (newd->f, ACS_VLINE, ACS_HLINE);
     mvwprintw (newd->f, 0, (cols-strlen (header)-2)/2, header);
@@ -110,7 +141,7 @@ int run_dialog ()
     push_event (1, 1, COLS, LINES, click_may_break_loop, quit_event);
     key = mi_getch ();
     pop_event ();
-    refresh_screen ();
+    do_refresh ();
     return key;
 }
 
@@ -190,6 +221,9 @@ char *input_dialog (char *header, char *text, char *def_text)
     Input  *in;
     int    c;
 
+    if (strlen (text) >= sizeof (buffer))
+	text [sizeof (buffer)-1] = 0;
+
     strcpy (buffer, text);
     
     len = max (strlen (header), msglen (buffer, &lines));
@@ -198,15 +232,30 @@ char *input_dialog (char *header, char *text, char *def_text)
     in = create_input (1, lines, current_dialog->t, INPUT_COLOR, 58, def_text);
     return_value = in->buffer;
     while ((c = mi_getch ()) != '\n'){
-	if (c == '\e'){
+	if (c == ESC_CHAR){
 	    return_value = 0;
 	    break;
 	}
 	handle_char (in, c);
     }
     destroy_dialog ();
-    (*refresh_fn)();
+    do_refresh ();
     destroy_input (in, IN_KEEP_BUFFER); 
     return (return_value);
 }
 
+char *input_expand_dialog (char *header, char *text, char *def_text)
+{
+    char *result;
+    char *expanded;
+
+    result = input_dialog (header, text, def_text);
+    if (result){
+	expanded = tilde_expand (result, no_gc);
+	if (result != expanded){
+	    free (result);
+	    return expanded;
+	}
+    }
+    return result;
+}
