@@ -13,6 +13,7 @@ typedef struct {
     struct { 
 	unsigned int marked:1;		/* File marked in pane window */
 	unsigned int exists:1;		/* Use for rereading file */
+	unsigned int link_to_dir:1;	/* If this is a link, does it point to directory? */
     } f;
 } file_entry;
 
@@ -22,11 +23,12 @@ typedef struct {
 } dir_list;
 
 typedef int sortfn (const void *, const void*);
-int do_load_dir (dir_list *list, sortfn *sort, int reverse);
+int do_load_dir (dir_list *list, sortfn *sort, int reverse, char *filter);
 void do_sort (dir_list *list, sortfn *sort, int top, int reverse);
 dir_list *do_collect_stat (dir_list *dir, int top);
-int do_reload_dir (dir_list *list, sortfn *sort, int count, int reverse);
+int do_reload_dir (dir_list *list, sortfn *sort, int count, int reverse, char *filter);
 void clean_dir (dir_list *list, int count);
+int set_zero_dir (dir_list *list);
 
 /* Sorting functions */
 int unsorted (const void *a, const void *b);
