@@ -21,6 +21,9 @@
 #include <errno.h>
 #include <ncurses.h>
 #include <malloc.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/param.h>
@@ -30,9 +33,7 @@
 #include "main.h"
 #include "util.h"
 
-static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/file.c,v 1.5 1994/08/02 04:39:46 miguel Exp $";
-static char x [4096];
-extern char *sys_errlist [];
+static char rcsid [] = "$Id: file.c,v 1.8 1994/10/13 04:27:31 miguel Exp $";
 int verbose = 1;
 
 void copy_file_file (char *src_path, char *dst_path)
@@ -48,12 +49,12 @@ void copy_file_file (char *src_path, char *dst_path)
     long n_read_total = 0;
 
     if ((source_desc = open (src_path, O_RDONLY)) < 0){
-	message (1, " Error ", sys_errlist [errno]);
+	message (1, " Error ", unix_error_string (errno));
 	return;
     }
 
     if (fstat (source_desc, &sb)){
-	message (1, " Error ", "%s: %s", sys_errlist [errno], dst_path);
+	message (1, " Error ", "%s: %s", unix_error_string [errno], dst_path);
 	goto ret;
     }
     src_mode = sb.st_mode;
@@ -62,19 +63,20 @@ void copy_file_file (char *src_path, char *dst_path)
        to not create a security hole.  */
 
     if ((dest_desc = open (dst_path, O_WRONLY | O_CREAT | O_TRUNC, 0600)) < 0){
-	message (1, " Error ", "cannot create regular file `%s'", dst_path);
+	message (1, " Error ", "cannot create regular file `%s' \n %s ",
+		 dst_path, unix_error_string (errno));
 	goto ret2;
     }
 
     /* Find out the optimal buffer size.  */
     if (fstat (dest_desc, &sb)){
-	message (1, " Error ", "%s: %s", sys_errlist [errno], dst_path);
+	message (1, " Error ", "%s: %s", unix_error_string (errno), dst_path);
 	goto ret;
     }
 
-    buf_size = 4*1024;
+    buf_size = 8*1024;
 
-    buf = (char *) alloca (buf_size + sizeof (int));
+    buf = (char *) xmalloc (buf_size, "copy_file_file");
 
     if (verbose){
 	message (D_INSERT, " Copying ",
@@ -86,7 +88,7 @@ void copy_file_file (char *src_path, char *dst_path)
     for (;;){
 	n_read = read (source_desc, buf, buf_size);
 	if (n_read < 0){
-	    message (1, " Error ", "%s: %s", sys_errlist [errno], src_path);
+	    message (1, " Error ", "%s: %s",unix_error_string(errno),src_path);
 	    goto ret;
 	}
 	if (n_read == 0)
@@ -96,13 +98,14 @@ void copy_file_file (char *src_path, char *dst_path)
 
 	n_written = write (dest_desc, buf, n_read);
 	if (n_written < n_read){
-	    message (1, " Error ", "%s: %s", sys_errlist [errno], dst_path);
+	    message (1, " Error ", "%s: %s", unix_error_string (errno),
+		     dst_path);
 	    goto ret;
 	}
     }
 
     if (chmod (dst_path, src_mode)){
-	    message (1, " Error ", "%s: chmod %s", sys_errlist [errno],
+	    message (1, " Error ", "%s: chmod %s", unix_error_string (errno),
 		     dst_path);
     }
     if (verbose){
@@ -110,12 +113,13 @@ void copy_file_file (char *src_path, char *dst_path)
 	repaint_screen (RP_NOCLEAR);
     }
  ret:
+    free (buf);
     if (close (dest_desc) < 0)
-	message (1, " Error ", "%s: %s", sys_errlist [errno], dst_path);
+	message (1, " Error ", "%s: %s", unix_error_string (errno), dst_path);
 
 ret2:
     if (close (source_desc) < 0)
-	message (1, " Error ", "%s: %s", sys_errlist [errno], src_path);
+	message (1, " Error ", "%s: %s", unix_error_string (errno), src_path);
 }
 
 void copy_file_dir (char *s, char *d)
@@ -127,8 +131,14 @@ void copy_file_dir (char *s, char *d)
 
 int move_file (char *s, char *d)
 {
-    sprintf (x, "/bin/mv %s %s", s, d);
+    char *x;
+    
+    x = copy_strings ("/bin/mv ", s, " ", d, 0);
+    open_error_pipe ();
     system (x);
+    close_error_pipe (1, NULL);
+
+    gc_free_now (x);
     return 1;
 }
 
@@ -147,7 +157,7 @@ int erase_file (char *s)
     error = unlink (s);
     if (verbose) destroy_dialog ();
     if (error){
-	message (1, " Error ", sys_errlist [errno]);
+	message (1, " Error ", unix_error_string (errno));
 	return 0;
     }
     return 1;
@@ -157,7 +167,7 @@ int erase_dir (char *s)
 {
     int error = rmdir (s);
     if (error){
-	message (1, " Error ", sys_errlist [errno]);
+	message (1, " Error ", unix_error_string (errno));
 	return 0;
     }
     return 1;
@@ -168,7 +178,8 @@ FILE *mopen (char *filename, char *mode)
     FILE *f;
     
     if ((f = fopen (filename, mode)) == NULL){
-	message (1, " Error ", " Can't open '%s' ", filename);
+	message (1, " Error ", " Can't open '%s' \n %s ",
+		 filename, unix_error_string (errno));
 	return 0;
     }
     return f;
