@@ -6,16 +6,16 @@
 @help_file = <>;
 
 foreach $line (@help_file){
-    if ($line =~ /\[(.*)\]/){
+    if ($line =~ /\x4\[(.*)\]/ && $line !~ /\x4\[main\]/){
 	$nodes[$node_count++] = $1;
     }
 }
 
-print "\x4[Index]\nTopics:\n\n";
-foreach $node (sort @nodes){
+print "\x4[Contents]\nTopics:\n\n";
+foreach $node (@nodes){
     print "\x1$node\x2$node\x3\n";
 }
-foreach $line (@help_file){
-    $line =~ s/%NEW_NODE%/\004/g;
-}
+#foreach $line (@help_file){
+#    $line =~ s/%NEW_NODE%/\004/g;
+#}
 print @help_file;
