@@ -9,3 +9,6 @@ void refresh_screen ();
 #ifdef ultrix
 char *strdup (char *);
 #endif
+
+/* AIX compiler doesn't understand '\e' */
+#define ESC_CHAR '\033'
