@@ -1,4 +1,4 @@
- /* Main program for the MouseLess Commander
+/* Main program for the Midnight Commander
    Copyright (C) 1994 Miguel de Icaza
    
    This program is free software; you can redistribute it and/or modify
@@ -14,8 +14,14 @@
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
-
 #include <ncurses.h>
+#ifndef TERM_INCLUDED
+#include <nterm.h>
+#endif
+/* Ugly hack to avoid name space pollution */
+#undef cols
+#undef lines
+#undef buttons
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -29,7 +35,10 @@
 #endif
 #include <errno.h>
 #include <pwd.h>
+#ifdef HAVE_UNISTD_H
+#include <sys/types.h>
 #include <unistd.h>
+#endif
 #include <fcntl.h>		/* For O_RDWR */
 #include <signal.h>
 
@@ -45,9 +54,11 @@
 #include "util.h"
 #include "win.h"
 #include "user.h"
+#include "mem.h"
 #include "mouse.h"
+#include "gc.h"
 
-static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/main.c,v 1.15 1994/09/08 19:28:12 miguel Exp $";
+static char rcsid [] = "$Id: main.c,v 1.19 1994/10/13 04:27:41 miguel Exp $";
 
 /* The structures for the panels */
 Panel left_panel, right_panel;
@@ -73,6 +84,9 @@ int   pause_after_run = 1;
 /* It true saves the setup when quitting */
 int auto_save_setup = 0;
 
+/* If true, on Linux Systems, auto-mount and auto-umount directories */
+int auto_mount = 0;
+
 /* If true, next character is quoted */
 int quote = 0;
 
@@ -91,7 +105,7 @@ char *prompt = "$ ";
 int force_slow = 0;
 
 /* use mouse? */
-int use_mouse_p = 1;
+int use_mouse_p = GPM_MOUSE;
 
 /* Controls screen clearing before an exec */
 int clear_before_exec = 1;
@@ -102,6 +116,10 @@ int confirm_delete = 1;
 /* Asks for confirmation before overwriting a file */
 int confirm_overwrite = 1;
 
+/* This flag is set by xterm detection routine in function main() */
+/* It is used by function view_other_cmd() */
+int xterm_flag = 0;
+
 /* WINDOW handles for the command line and the function keys */
 WINDOW *cmdline_win;
 WINDOW *fkeys;
@@ -128,11 +146,28 @@ int is_right;
 char search_buffer [256] = { 0 };
 char cmd_buf [512];
 
-extern char *sys_errlist [];
-
 void save_setup_cmd ();
 void menu_cmd ();
 
+static int status_mouse_support = 
+#ifdef HAVE_LIBGPM
+    1;
+#else
+    0;
+#endif
+static int status_using_ncurses = 
+#ifdef OTHER_CURSES
+    0;
+#else
+    1;
+#endif
+static int status_using_old_tools =
+#ifdef OLD_TOOLS
+    1;
+#else
+    0;
+#endif
+
 inline void unselect_item (Panel *panel)
 {
     set_attr (cpanel, 0, selection->f.marked);
@@ -153,30 +188,40 @@ inline void select_item (Panel *panel)
     display_mini_info (cpanel);
 }
 
-#ifdef use_mouse
 int panel_event    (Gpm_Event *event, Panel *panel);
 int menu_bar_event (Gpm_Event *event, void *);
-#endif
+
 void init_panels ()
 {
     init_panel (&left_panel, 0, 0, COLS/2, LINES-2);
     left_panel.active = 1;
     left_panel.count  = do_load_dir (&left_panel.dir, left_panel.sort_type,
-				     left_panel.reverse);
+				     left_panel.reverse, left_panel.filter);
     if (other_dir)
 	chdir (other_dir);
-    init_panel (&right_panel, COLS/2, 0, COLS, LINES-2);
-    right_panel.count = do_load_dir (&right_panel.dir, right_panel.sort_type,
-				     right_panel.reverse);
-    if (other_dir)
-	chdir (left_panel.cwd);
-
-    current_panel = &left_panel;
-    other_panel   = &right_panel;
     
-    select_item (current_panel);/* Inits selection variable */
-    display_mini_info (cpanel);	/* It needs selection defined */
-    display_mini_info (opanel);	/* Same here */
+    init_panel (&right_panel, COLS/2, 0, COLS, LINES-2);
+    right_panel.count = do_load_dir (&right_panel.dir, right_panel.sort_type, 
+				     right_panel.reverse, right_panel.filter);
+
+    /* Don't select the information window */
+    if (left_panel.view_type == view_info){
+	cpanel = &right_panel;
+	opanel   = &left_panel;
+    } else {
+	cpanel = &left_panel;
+	opanel   = &right_panel;
+	if (other_dir)
+	    chdir (left_panel.cwd);
+    }
+    cpanel->active = 1;
+    opanel->active = 0;
+
+    select_item (cpanel);	/* Inits selection variable */
+
+    display_mini_info (cpanel); /* It needs selection defined */
+
+    display_mini_info (opanel); /* Same here */
 
     /* Now, update the screen */
     paint_panel (&left_panel);
@@ -263,9 +308,10 @@ void pre_exec ()
 /* Restores the terminal after an exec, see execute for extra post-exec code */
 void post_exec ()
 {
+    putp (enter_ca_mode);
     reset_prog_mode ();
     flushinp ();
-    cbreak ();
+    raw ();
     keypad (stdscr, TRUE);
     if (use_mouse_p)
 	init_mouse ();
@@ -283,29 +329,19 @@ void save_cwds_stat ()
 
 void execute (char *command)
 {
-#ifdef OLD_SYSTEM_CODE
-    char buf [4096];
-#endif
     struct passwd *pw_data;
 
     pw_data = getpwuid (geteuid ());
-
-#ifdef OLD_SYSTEM_CODE
-    sprintf (buf, "%s -c \"%s\"", pw_data->pw_shell, command);
-#endif
-    
     save_cwds_stat ();
     pre_exec ();
-#ifdef OLD_SYSTEM_CODE
-    system (buf);
-#else
     my_system (pw_data->pw_shell, command);
-#endif
     if (pause_after_run){
 	printf ("Press any key to continue...");
 	fflush (stdout);
-	cbreak ();
+	raw ();
 	mi_getch ();
+	printf ("\r\n");
+	fflush (stdout);
     }
     post_exec ();
     update_panels (UP_OPTIMIZE, UP_KEEPSEL, 0);
@@ -331,7 +367,8 @@ void change_panel ()
 	current_panel = &left_panel;
     current_panel->active = 1;
     if (chdir (current_panel->cwd) != 0){
-	message (1, " Error ", " Can't chdir to %s ", current_panel->cwd);
+	message (1, " Error ", " Can't chdir to %s \n %s ",
+		 current_panel->cwd, unix_error_string (errno));
     }
     show_dir (current_panel);
     select_item (current_panel);
@@ -339,7 +376,7 @@ void change_panel ()
 
 int quit_cmd ()
 {
-    if (query_dialog (" The MouseLess Commander  ",
+    if (query_dialog (" The Midnight Commander  ",
 		      " Do you really want to quit the Mouse-less commander? ",
 		      0, 2, " Yes ", " No ") == 0)
 	quit = 1;
@@ -476,8 +513,12 @@ void untouch_bar ()
 {
     touchwin (cpanel->win_file);
     touchwin (opanel->win_file);
+#ifdef BUGGY_CURSES
+    /* On Solaris the provided wtouchln in curses doesn't work */
+#else
     wtouchln (cpanel->win_file, 0, 1, 0);
     wtouchln (opanel->win_file, 0, 1, 0);
+#endif
     panel_refresh (cpanel);
     panel_refresh (opanel);
 }
@@ -531,7 +572,7 @@ void refresh_screen ()
 
 void help_cmd ()
 {
-    interactive_display (HELPFILE);
+    interactive_display (LIBDIR "mc.hlp", "[main]");
 }
 
 int do_cd (char *new_dir)
@@ -540,7 +581,10 @@ int do_cd (char *new_dir)
     DIR  *dirp;
     char *directory;
     char temp [MAXPATHLEN];
-	
+
+    while (*new_dir == ' ')
+	new_dir++;
+    
     if (!strcmp (new_dir, "-")){
 	strcpy (temp, cpanel->lwd);
 	new_dir = temp;
@@ -550,23 +594,40 @@ int do_cd (char *new_dir)
     }
     
     directory = *new_dir ? new_dir : home_dir;
+
+    /* If it requires expansion */
+
+    directory = tilde_expand (directory, use_gc);
+    gc_free (directory);	/* Schedule free for next loop */
+    
     dirp = opendir (directory);
     if (!dirp)
 	return 0;
     closedir (dirp);
 
+    if (auto_mount)
+	try_auto_mount (directory);
+
     ret = chdir (directory);
+    
+    if (auto_mount)
+	try_auto_umount (cpanel->cwd);
+
     if (ret == -1)
 	return 0;
     
     clean_dir (&cpanel->dir, cpanel->count);
     cpanel->count = do_load_dir (&cpanel->dir, cpanel->sort_type,
-				 cpanel->reverse);
+				 cpanel->reverse, cpanel->filter);
     cpanel->top_file = 0;
     cpanel->selected = 0;
     cpanel->marked = 0;
     cpanel->total = 0;
+#ifdef HAVE_GETWD
     getwd (cpanel->cwd);
+#else
+    getcwd (cpanel->cwd, sizeof (cpanel->cwd)-2);
+#endif
     return 1;
 }
 
@@ -588,8 +649,9 @@ void action ()
 	return;
     }
     if (is_exe (selection->buf.st_mode) && if_link_is_exe (selection)){
-	/* Need to append "./" to the command */
-	execute (selection->fname);
+	char *tmp = copy_strings ("./", selection->fname, 0);
+	execute (tmp);
+	gc_free (tmp);
 	return;
     }
     regex_command (selection->fname);
@@ -603,8 +665,8 @@ void enter ()
 	    if (cmdline->buffer [2] == 0)
 		cmdline->buffer [3] = 0;
 	    if (!do_cd (&cmdline->buffer [3])){
-		message (1, " Error ", " Can't chdir to '%s' ",
-			 &cmdline->buffer [3]);
+		message (1, " Error ", " Can't chdir to '%s' \n %s ",
+			 &cmdline->buffer [3], unix_error_string (errno));
 		return;
 	    }
 	    paint_panel (cpanel);
@@ -627,9 +689,11 @@ void do_search (int c_code)
     if (l && (c_code == 8 || c_code == 0177 || c_code == KEY_BACKSPACE))
 	search_buffer [--l] = 0;
     else {
-	search_buffer [l] = c_code & 0x7f;
-	search_buffer [l+1] = 0;
-	l++;
+	if (l < sizeof (search_buffer)){
+	    search_buffer [l] = c_code & 0x7f;
+	    search_buffer [l+1] = 0;
+	    l++;
+	}
     }
     for (i = 0; i < cpanel->count; i++){
 	if (strncmp (cpanel->dir.list [i].fname, search_buffer, l) == 0){
@@ -651,8 +715,6 @@ void start_search ()
     search_buffer [0] = 0;
 }
 
-#ifdef use_mouse
-
 static int mouse_marking = 0;
 
 void mouse_toggle_mark ()
@@ -701,9 +763,17 @@ int panel_event (Gpm_Event *event, Panel *panel)
 	    mark_if_marking (event);
 	    move_down ();
 	    return MOU_REPEAT;
-	} else
+	} else {
 	    index = panel->top_file + event->y - 1;
+	    if (panel->view_type == view_brief){
+		if (event->x > (panel->cols/2))
+		    index += panel->lines;
+	    }
+	}
 
+	if (index >= panel->count)
+	    index = panel->count - 1;
+	
 	if (index != panel->selected){
 	    unselect_item (panel);
 	    panel->selected = index;
@@ -723,7 +793,6 @@ int menu_bar_event (Gpm_Event *event, void *x)
     menu_cmd ();
     return MOU_NORMAL;
 }
-#endif
 
 /* If the key pressed is not bound to any command, this function is called */
 void default_key (int c_code)
@@ -735,13 +804,66 @@ void default_key (int c_code)
     }
 }
 
+/* Changes the selection by lines (may be negative) */
+void move_selection (Panel *panel, int lines)
+{
+    int new_pos;
+    int adjust;
+
+    new_pos = panel->selected + lines;
+    if (new_pos >= panel->count)
+	new_pos = panel->count-1;
+
+    if (new_pos < 0)
+	new_pos = 0;
+
+    unselect_item (panel);
+    panel->selected = new_pos;
+
+    if (panel->selected - panel->top_file >= ITEMS (panel)){
+	panel->top_file += lines;
+	adjust = 1;
+    }
+    
+    if (panel->selected - panel->top_file < 0){
+	panel->top_file += lines;
+	adjust = 1;
+    }
+    
+    if (adjust){
+	if (panel->top_file > panel->selected)
+	    panel->top_file = panel->selected;
+	if (panel->top_file < 0)
+	    panel->top_file = 0;
+	paint_dir (panel);
+    }
+    select_item (panel);
+    panel_refresh (panel);
+}
+
+void move_left (int c_code)
+{
+    if (cpanel->view_type == view_brief)
+	move_selection (cpanel, -cpanel->lines);
+    else
+	default_key (c_code);
+}
+
+void move_right (int c_code)
+{
+    if (cpanel->view_type == view_brief)
+	move_selection (cpanel, cpanel->lines);
+    else
+	default_key (c_code);
+}
+
 void view_cmd ()
 {
     static char *viewer = 0;
+    char *command;
 
     if (use_internal_view){
 	view (selection->fname);
-	refresh_fn = refresh_screen;
 	repaint_screen (RP_NOCLEAR);
     } else {
 	if (!viewer){
@@ -749,8 +871,9 @@ void view_cmd ()
 	    if (!viewer)
 		viewer = "view";
 	}
-	sprintf (cmd_buf, "%s %s", viewer, selection->fname);
-	execute (cmd_buf);
+	command = copy_strings (viewer, " ", selection->fname, 0);
+	execute (command);
+	gc_free (command);
     }
 }
 
@@ -859,10 +982,10 @@ void copy_cmd ()
 	sprintf (cmd_buf," Copy \"%s\" file to:",
 		 name_trunc(selection->fname, 30));
     }
-    dest = input_dialog (" Copy ", cmd_buf, other_panel->cwd);
+    dest = input_expand_dialog (" Copy ", cmd_buf, other_panel->cwd);
     if (!dest)
 	return;
-    
+
     stat_r = stat (dest, &s);
     save_cwds_stat ();
     if (S_ISREG (s.st_mode) || stat_r != 0){
@@ -906,7 +1029,7 @@ void copy_cmd ()
 		    cpanel->total -= cpanel->dir.list [i].buf.st_size;
 		}
 		if ((c = getch ()) != ERR)
-		    if (c == '\e' || c == 3)
+		    if (c == ESC_CHAR || c == XCTRL('c'))
 			break;
 	    }
 	    nodelay (stdscr, FALSE);
@@ -922,8 +1045,8 @@ void ren_cmd ()
     struct stat s;
     int i, c, stat_r;
     
-    char *new_name = input_dialog (" Rename/Move ", "Enter rename mask:",
-				   opanel->cwd);
+    char *new_name = input_expand_dialog (" Rename/Move ",
+					  "Enter rename mask:", opanel->cwd);
     if (!new_name)
 	return;
     if (strchr (new_name, '*') || strchr (new_name, '?')){
@@ -957,7 +1080,7 @@ void ren_cmd ()
 		    move_file_dir (cpanel->dir.list [i].fname, new_name);
 		}
 		if ((c = getch ()) != ERR)
-		    if (c == '\e' || c == 3)
+		    if (c == ESC_CHAR || c == XCTRL('c'))
 			break;
 	    }
 	    nodelay (stdscr, FALSE);
@@ -983,7 +1106,8 @@ void do_link (int symbolic_link)
     if (!symbolic_link){
 	stat_r = stat (selection->fname, &s);
 	if (stat_r != 0){
-	    message (1, " Error ", " Couldn't stat %s ", selection->fname);
+	    message (1, " Error ", " Couldn't stat %s \n %s ",
+		     selection->fname, unix_error_string (errno));
 	    return;
 	}
 	if (!S_ISREG (s.st_mode))
@@ -991,17 +1115,18 @@ void do_link (int symbolic_link)
     }
     
     if (!symbolic_link){
-	dest = input_dialog (" Link ", " Link file to: ", "");
+	dest = input_expand_dialog (" Link ", " Link file to: ", "");
 	if (!dest)
 	    return;
 	save_cwds_stat ();
 	if (-1 == link (selection->fname, dest))
 	    errno_report ("link");
     } else {
-	src = input_dialog (" Symlink ", "Symlink file: ", "");
+	src = input_expand_dialog (" Symlink ", "Symlink file: ", "");
 	if (!src)
 	    return;
-	dest = input_dialog (" Symlink ", " -> To file: ", selection->fname);
+	dest = input_expand_dialog (" Symlink ", " -> To file: ",
+				    selection->fname);
 	if (src){
 	    save_cwds_stat ();
 	    if (-1 == symlink (dest, src))
@@ -1026,15 +1151,13 @@ void symlink_cmd ()
 
 void mkdir_cmd ()
 {
-    char *dir = input_dialog (" Mkdir ", "Enter directoy name:" , "");
+    char *dir = input_expand_dialog (" Mkdir ", "Enter directory name:" , "");
     if (!dir)
 	return;
 
     save_cwds_stat ();
     if (mkdir (dir, 0777) == 0){
-	update_panels (UP_OPTIMIZE, 0, 0);
-	unselect_item (cpanel);
-	try_to_select (cpanel, dir);
+	update_panels (UP_OPTIMIZE, dir, 0);
 	repaint_screen (RP_NOCLEAR);
 	select_item (cpanel);
 	free (dir);
@@ -1080,17 +1203,90 @@ void find_cmd ()
     do_find ();
 }
 
+void swap_cmd ()
+{
+    Panel temp1, *temp2;
+    WINDOW *temp3;
+
+    temp1 = left_panel;
+    left_panel = right_panel;
+    right_panel = temp1;
+
+    temp2 = current_panel;
+    current_panel = other_panel;
+    other_panel = temp2;
+
+    temp3 = left_panel.win_file;
+    left_panel.win_file = right_panel.win_file;
+    right_panel.win_file = temp3;
+    temp3 = left_panel.win_mini;
+    left_panel.win_mini = right_panel.win_mini;
+    right_panel.win_mini = temp3;
+    temp3 = left_panel.small_frame;
+    left_panel.small_frame = right_panel.small_frame;
+    right_panel.small_frame = temp3;
+    temp3 = left_panel.big_frame;
+    left_panel.big_frame = right_panel.big_frame;
+    right_panel.big_frame = temp3;
+
+    paint_panel (current_panel);
+    paint_panel (other_panel);
+}
+
+void view_other_cmd ()
+{
+#ifdef OTHER_CURSES
+    message (1, " Error ", " %s \n %s ",
+	     "Using some unknown curses",
+	     "The View other screen command is not supported");
+#else
+    if (xterm_flag){
+	if (use_mouse_p)
+	  shut_mouse ();
+	reset_shell_mode ();
+	endwin ();
+	raw ();
+	noecho ();
+	mi_getch ();
+	putp (enter_ca_mode);
+	reset_prog_mode ();
+	if (use_mouse_p)
+	  init_mouse ();
+	refresh ();
+    } else
+        message (1, " Error ", " Not a xterm ");
+#endif /* OTHER_CURSES */
+}
+
+/* Check if the file exists */
+/* If not copy the default */
+int check_for_default(char *default_file, char *file)
+{
+    struct stat s;
+    if (stat (file, &s)){
+	if (stat (default_file, &s)){
+	    return -1;
+	}
+	copy_file_file (default_file, file);
+    }
+    return 0;
+}
+
 void ext_cmd ()
 {
-    char buffer [400];
-    
-    sprintf (buffer, "%s/.mc.ext", home_dir);
+    char *buffer;
+
+    buffer = copy_strings (home_dir, "/.mc.ext", 0);
+    check_for_default (LIBDIR "mc.ext", buffer);
     do_edit (buffer);
+    gc_free (buffer);
 }
 
 void menu_edit_cmd ()
 {
-    char buffer [400];
+    char *buffer, *home_file;
+#define MC_MENU "mc.menu"
+
     int dir;
     
     if (getuid () == 0)
@@ -1101,23 +1297,30 @@ void menu_edit_cmd ()
 	dir = query_dialog ("Menu file edit",
 			   " Which menu file will you edit? ", 0, 2,
 			   " Local ", " User ");
+    home_file = copy_strings (home_dir, "/.mc.menu", 0);
+
     switch (dir){
     case 0:
-	strcpy (buffer, "/.mc.menu");
+	buffer = ".mc.menu";
+	if (check_for_default (home_file, buffer))
+	    check_for_default (LIBDIR MC_MENU, buffer);
 	break;
 
     case 1:
-	sprintf (buffer, "%s/.mc.menu", home_dir);
+	buffer = home_file;
+	check_for_default (LIBDIR MC_MENU, buffer);
 	break;
 
     case 2:
-	strcpy (buffer, MENU);
+	buffer = copy_strings (LIBDIR, MC_MENU, 0);
 	break;
 
     default:
 	return;
     }
     do_edit (buffer);
+    gc_free (buffer);
+    gc_free (home_file);
 }
 
 void select_cmd ()
@@ -1132,7 +1335,7 @@ void select_cmd ()
     for (i = 0; i < cpanel->count; i++){
 	if (S_ISDIR (cpanel->dir.list [i].buf.st_mode))
 	    continue;
-	c = regexp_match (reg_exp, cpanel->dir.list [i].fname);
+	c = regexp_match (reg_exp, cpanel->dir.list [i].fname, match_file);
 	if (c == -1){
 	    message (1, " Error ", "  Malformed regular expression  ");
 	    free (reg_exp);
@@ -1164,7 +1367,7 @@ void unselect_cmd ()
     for (i = 0; i < cpanel->count; i++){
 	if (S_ISDIR (cpanel->dir.list [i].buf.st_mode))
 	    continue;
-	c = regexp_match (reg_exp, cpanel->dir.list [i].fname);
+	c = regexp_match (reg_exp, cpanel->dir.list [i].fname, match_file);
 	if (c == -1){
 	    message (1, " Error ", "  Malformed regular expression  ");
 	    free (reg_exp);
@@ -1214,9 +1417,45 @@ void brief_cmd ()
     paint_panel (p);
 }
 
+void paint_info_panel (Panel *panel)
+{
+    WINDOW *w = panel->win_file;
+
+    set_colors (panel);
+    wclrn (w, 3);
+    wmove (w, 2, 2);
+    mvwprintw (w, 2, 3, "Midnight Commander %g", VERSION);
+    mvwprintw (w, 2, 4, "");
+    wmove (w, 5, 1);
+    whline (w, ACS_HLINE|NORMAL_COLOR, panel->cols);
+}
+
+/* Switches panel p to info mode */
+static void switch_to_info (Panel *p, Panel *o)
+{
+    if ((o->view_type & VIEW_DISABLED)||(o->view_type == view_info))
+	return;
+
+    if (p->view_type == view_info)
+	full_cmd (p);
+    else {
+	p->view_type = view_info;
+	info_frame (p);
+	paint_panel (p);
+    }
+}
+
 void info_cmd ()
 {
-    
+    Panel *p = MENU_PANEL;
+    Panel *o = OTHER_PANEL;
+
+    switch_to_info (p, o);
+}
+
+void info_cmd_no_menu ()
+{
+    switch_to_info (opanel, cpanel);
 }
 
 /* Switches the panel to long display (like ls -l) */
@@ -1304,11 +1543,31 @@ void reread_cmd ()
     repaint_screen (RP_NOCLEAR);
 }
 
+void filter_cmd ()
+{
+    char *reg_exp;
+    char *x;
+    Panel *p = MENU_PANEL;
+
+    x = p->filter ? p->filter : "*";
+	
+    reg_exp = input_dialog (" Filter ", "", x);
+    if (!reg_exp)
+	return;
+    if (p->filter){
+	free (p->filter);
+	p->filter = 0;
+    }
+    if (!(reg_exp [0] == '*' && reg_exp [1] == 0))
+	p->filter = reg_exp;
+    reread_cmd ();
+}
+
 menu_entry PanelMenu [] = {
     "  Full",  'F', full_cmd,
     "  Brief", 'B', brief_cmd,
-    "  Long",  'L', long_cmd, 
-/*    "  Info",  'I', info_cmd, */
+    "  lonG",  'G', long_cmd, 
+    "  Info",  'I', info_cmd,
     "", ' ', 0,
     "  Name",      'N', by_name_cmd,
     "  Extension", 'E', by_ext_cmd,
@@ -1318,13 +1577,14 @@ menu_entry PanelMenu [] = {
     "  reVerse ",  'V', reverse_cmd,
     "", ' ', 0,
     "  Reread    C-r",XCTRL('R'), reread_cmd,
-/*    "  Filter...", 'l', reread_cmd, */
+    "  fiLter...", 'L', filter_cmd,
 };
 
 menu_entry RightMenu [] = {
     "  Full",  'F', full_cmd,
     "  Brief", 'B', brief_cmd,
-    "  Long",  'L', long_cmd, 
+    "  lonG",  'G', long_cmd, 
+    "  Info",  'I', info_cmd,
     "", ' ', 0,
     "  Name",      'N', by_name_cmd,
     "  Extension", 'E', by_ext_cmd,
@@ -1334,9 +1594,7 @@ menu_entry RightMenu [] = {
     "  reVerse ",  'V', reverse_cmd,
     "", ' ', 0,
     "  Reread    C-r",    XCTRL('R'), reread_cmd,
-    /*
-    "  Filter...", 'l', reread_cmd,
-    */
+    "  fiLter...", 'L', filter_cmd,
 };
 
 menu_entry FileMenu [] = {
@@ -1358,10 +1616,14 @@ menu_entry FileMenu [] = {
 };
 
 menu_entry CmdMenu [] = {
-    "Find file (test)", 'F', find_cmd,
-    "Save setup",       'S', save_setup_cmd,
-    "Extension file edit", 'E', ext_cmd,
-    "Menu file edit",   'M', menu_edit_cmd,
+    "Find file (test)",         'F', find_cmd,
+    "Swap panels C-u",		XCTRL('u'), swap_cmd,
+    "Panels on/off(xterm) C-o", XCTRL('o'), view_other_cmd,
+    "", ' ', 0, 
+    "Extension file edit",      'E', ext_cmd,
+    "Menu file edit",           'M', menu_edit_cmd,
+    "", ' ', 0,
+    "Save setup",               'S', save_setup_cmd,
 };
 
 /* Must keep in sync with the constants in menu_cmd */
@@ -1379,6 +1641,7 @@ menu_entry OptMenu [] = {
     "  miX all files",     'X', toggle_mix_all_files,
     "  Fast dir-reload",   'F', toggle_fast_reload,
     "  Confirm delete",    'C', toggle_confirm_delete,
+    "  automount/Umount C-x a",  'U', toggle_auto_mount,
 /*    "  Align extensions",  'A', toggle_align_extensions, */
 };
 
@@ -1395,9 +1658,9 @@ void init_menu ()
     MenuBar [4] = create_menu (" Right ", RightMenu, menu_entries (PanelMenu));
 }
 
-void check_options_menu (int index, int flag)
+void check_options_menu (int mindex, int flag)
 {
-    *OptMenu [index].text = flag ? '*' : ' ';
+    *OptMenu [mindex].text = flag ? '*' : ' ';
 }
 
 void check_menu_panel (Panel *panel, menu_entry PanelMenu [])
@@ -1405,14 +1668,14 @@ void check_menu_panel (Panel *panel, menu_entry PanelMenu [])
     PanelMenu [0].text [0] = panel->view_type == view_full ? '*' : ' ';
     PanelMenu [1].text [0] = panel->view_type == view_brief ? '*' : ' ';
     PanelMenu [2].text [0] = panel->view_type == view_long ? '*' : ' ';
-/*    PanelMenu [3].text [0] = panel->view_type == view_info ? '*' : ' '; */
+    PanelMenu [3].text [0] = panel->view_type == view_info ? '*' : ' ';
 
-    PanelMenu[4].text[0]= panel->sort_type == (sortfn *)sort_name ? '*' : ' ';
-    PanelMenu[5].text[0]= panel->sort_type == (sortfn *) sort_ext ? '*' : ' ';
-    PanelMenu[6].text[0]= panel->sort_type == (sortfn *) sort_time ? '*' : ' ';
-    PanelMenu[7].text[0]= panel->sort_type == (sortfn *) sort_size ? '*' : ' ';
-    PanelMenu[8].text[0]= panel->sort_type == (sortfn *) unsorted ? '*' : ' ';
-    PanelMenu[9].text[0]= panel->reverse == 1 ? '*' : ' ';
+    PanelMenu[5].text[0]= panel->sort_type == (sortfn *)sort_name ? '*' : ' ';
+    PanelMenu[6].text[0]= panel->sort_type == (sortfn *) sort_ext ? '*' : ' ';
+    PanelMenu[7].text[0]= panel->sort_type == (sortfn *) sort_time ? '*' : ' ';
+    PanelMenu[8].text[0]= panel->sort_type == (sortfn *) sort_size ? '*' : ' ';
+    PanelMenu[9].text[0]= panel->sort_type == (sortfn *) unsorted ? '*' : ' ';
+    PanelMenu[10].text[0]= panel->reverse == 1 ? '*' : ' ';
 }
 
 void menu_cmd ()
@@ -1432,19 +1695,25 @@ void menu_cmd ()
     check_options_menu (10, mix_all_files);
     check_options_menu (11, fast_reload);
     check_options_menu (12, confirm_delete);
-    
+    check_options_menu (13, auto_mount);
+
     check_menu_panel (&left_panel, PanelMenu);
     check_menu_panel (&right_panel, RightMenu);
     panel = current_panel == &right_panel ? 4 : 0;
 
     push_frame (0, 0, 0);
-    run_bar (stdscr, 0, 0, 80, 12, 5, panel, MenuBar, BAR_FILL, A_BOLD,
+    run_bar (stdscr, 0, 0, COLS, 12, 5, panel, MenuBar, BAR_FILL, A_BOLD,
 	     SELECTED_COLOR);
     pop_frame ();
 }
 
 /* Flag toggling functions */
 
+void toggle_auto_mount ()
+{
+    auto_mount = !auto_mount;
+}
+
 void toggle_confirm_delete ()
 {
     confirm_delete = !confirm_delete;
@@ -1609,6 +1878,9 @@ void copy_current_pathname ()
 
 void copy_other_pathname ()
 {
+    if ((opanel->view_type==view_info) || (opanel->view_type & VIEW_DISABLED))
+	return;
+
     stuff (cmdline, opanel->cwd, 0);
 }
 
@@ -1631,9 +1903,22 @@ void copy_current_tagged ()
 
 void copy_other_tagged ()
 {
+    if ((opanel->view_type==view_info) || (opanel->view_type & VIEW_DISABLED))
+	return;
+
     copy_tagged (opanel);
 }
 
+void mount_this_dir ()
+{
+    try_auto_mount (selection->fname);
+}
+
+void umount_this_dir ()
+{
+    try_auto_umount (selection->fname);
+}
+
 void suspend_cmd ()
 {
     save_cwds_stat ();
@@ -1658,10 +1943,14 @@ key_map ctl_x_map [] = {
     'l',          link_cmd,
     's',          symlink_cmd,
     'o',          change_panel,
+    'a',          toggle_auto_mount,
+    'm',          mount_this_dir,
+    'u',          umount_this_dir,
+    'i',          info_cmd_no_menu,
     0, 0,
 };
 
-void ctl_x_cmd ()
+void ctl_x_cmd (int ignore)
 {
     int i;
     int key = mi_getch ();
@@ -1675,23 +1964,16 @@ void ctl_x_cmd ()
 }
 
 key_map default_map [] = {
-    KEY_F(10), 	(callfn) quit_cmd,
-    KEY_F(1),   help_cmd,
-    KEY_F(2),   user_menu_cmd, 
-    KEY_F(3),   view_cmd,
-    KEY_F(4),   edit_cmd,
-    KEY_F(5),   copy_cmd,
-    KEY_F(6),   ren_cmd,
-    KEY_F(7),   mkdir_cmd,
-    KEY_F(8),   delete_cmd,
-    KEY_F(9),   menu_cmd,
     KEY_F(12),  dump_marked,
     '+',        select_cmd,
     '\\',       unselect_cmd,
     '\t',  	change_panel,
     '\n',       enter,		/* Enter */
+    
     KEY_DOWN,   move_down,
     KEY_UP, 	move_up,
+    KEY_LEFT,   move_left,
+    KEY_RIGHT,  move_right,
     KEY_IC,     mark_file,
     KEY_HOME,	move_home,
     KEY_C1,     move_end,
@@ -1729,6 +2011,12 @@ key_map default_map [] = {
     /* Panel refresh */
     XCTRL('r'), reread_cmd,
 
+    /* Swap panels */
+    XCTRL('u'), swap_cmd,
+
+    /* View output */
+    XCTRL('o'), view_other_cmd,
+    
     /* Control-X keybindings */
     XCTRL('x'), ctl_x_cmd,
     
@@ -1738,7 +2026,7 @@ key_map default_map [] = {
 
 void init_labels ()
 {
-    fkeys   = new_fkey ();
+    fkeys   = push_fkey ();
     define_label (fkeys, 1, "Help", help_cmd);
     define_label (fkeys, 2, "Menu", user_menu_cmd);
     define_label (fkeys, 3, "View", view_cmd);
@@ -1768,9 +2056,9 @@ void do_nc ()
     int key;
     int i;
 
-    refresh_fn = refresh_screen;
+    push_refresh (refresh_screen);
     if (COLS < 70 || LINES < 12){
-	fprintf (stderr, "Screen too small: you need at leas 70x12\n");
+	fprintf (stderr, "Screen too small: you need at least 70x12\n");
 	return;
     }
 
@@ -1800,17 +2088,24 @@ void do_nc ()
 
     /* Main program loop */
     for (quit = 0; !quit;){
+	garbage_collect ();
 	key = mi_getch ();
 	if (quote){
 	    default_key (key);
 	    quote = 0;
 	    continue;
 	}
-	for (i = 0; default_map [i].key_code; i++){
-	    if (key == default_map [i].key_code){
-		searching = 0;
-		(*default_map [i].fn)(key);
-		break;
+	if (check_fkeys (key)){
+	    searching = 0;
+	    continue;
+	}
+	else{
+	    for (i = 0; default_map [i].key_code; i++){
+		if (key == default_map [i].key_code){
+		    searching = 0;
+		    (*default_map [i].fn)(key);
+		    break;
+		}
 	    }
 	}
 	if (default_map [i].key_code)
@@ -1820,6 +2115,7 @@ void do_nc ()
 	(*default_map [i].fn)(key);
 	wrefresh (cmdline_win);
     }
+    pop_fkey (fkeys);
     destroy_input (cmdline, IN_NORMAL);
     if (auto_save_setup)
 	save_setup ();
@@ -1834,14 +2130,12 @@ void do_nc ()
 
 void version ()
 {
-    int mouse_support = 
-#ifdef use_mouse
-    1;
-#else
-    0;
-#endif
-    printf ("MouseLess Commander %s %s\n", VERSION,
-	    mouse_support ? "with mouse support on Linux console" : "");
+    printf ("Midnight Commander %g %s %s\n%s %s\n", VERSION,
+	    "with mouse support on 'xterm' terminal",
+	    status_mouse_support ? "and on Linux console" : "",
+	    status_using_ncurses ?"using ncurses":"using some unknown curses",
+	    status_using_old_tools ? "and was compiled with old tools" : ""
+	    );
 }
 
 int main (int argc, char *argv [])
@@ -1850,6 +2144,13 @@ int main (int argc, char *argv [])
     char   *this_dir = 0;
     int    c;
     int    output1, output2;
+    char   *termvalue;
+
+    termvalue = getenv ("TERM");
+    if (strncmp (termvalue, "xterm", 5) == 0){
+	use_mouse_p = XTERM_MOUSE;
+	xterm_flag = 1;
+    }
 
     while ((c = getopt (argc, argv, "VbsdPcf")) != -1){
 	switch (c){
@@ -1867,10 +2168,10 @@ int main (int argc, char *argv [])
 
 	case 'f':
 	    printf (
-		    "Help file:            " HELPFILE "\n"
-	            "Global defaults file: " PROFILE "\n"
-		    "Global user menu:     " MENU "\n"
-		    "Extensions file:      " EXT "\n");
+		    "Help file:            " LIBDIR "mc.hlp" "\n"
+	            "Global defaults file: " LIBDIR "mc.ini" "\n"
+		    "Global user menu:     " LIBDIR "mc.menu" "\n"
+		    "Extensions file:      " LIBDIR "mc.ext" "\n");
 	    exit (1);
 
 	case 'P':
@@ -1882,13 +2183,13 @@ int main (int argc, char *argv [])
 	    break;
 
 	case 'd':
-	    use_mouse_p = 0;
+	    use_mouse_p = NO_MOUSE;
 	    break;
 
 	default:
 	    version ();
 	    printf ("Usage is:\n"
-		    "mc [-bdPsV] [other_panel_dir]\n"
+		    "mc [-bcdfPsV] [other_panel_dir]\n"
 		    "-b Force black and white\n"
 		    "-c Force color, only available if terminfo allows it\n"
 		    "-d Disable mouse support\n"
@@ -1910,19 +2211,27 @@ int main (int argc, char *argv [])
 	    }
 	} else {
 	    this_dir = argv [optind];
+#ifdef HAVE_GETWD
 	    getwd (right_panel.cwd);
+#else
+	    getcwd (right_panel.cwd, sizeof (right_panel.cwd)-2);
+#endif
 	    other_dir = right_panel.cwd;
 	}
     }
     if (this_dir)
 	chdir (this_dir);
+
+    /* Used to report the las working directory at program end */
     if (print_last_wd){
 	output1 = dup (1);
 	close (1);
 	output2 = open ("/dev/tty", O_RDWR);
     }
+
+    /* We need this, since ncurses endwin () doesn't restore the signals */
     save_stop_handler ();
-    printf ("\ec");
+
     initscr ();
     do_nc ();
     endwin ();
