@@ -1,3 +1,4 @@
+/* Toggling functions */
 void toggle_clean_exec ();
 void toggle_show_backup ();
 void toggle_show_hidden ();
@@ -13,16 +14,22 @@ void toggle_internal ();
 void toggle_mix_all_files ();
 void toggle_fast_reload ();
 void toggle_confirm_delete ();
+void toggle_auto_mount ();
 
 void do_mark_file (int do_move);
 
-#define RP_CLEAR 1
-#define RP_NOCLEAR 0
+enum {
+    RP_NOCLEAR,
+    RP_CLEAR
+};
 
-#define UP_OPTIMIZE 0
-#define UP_RELOAD   1
+enum {
+    UP_OPTIMIZE = 0,
+    UP_RELOAD   = 1,
+} update_type;
 
 #define UP_KEEPSEL (char *) -1
+
 extern int quote;
 extern int quit;
 void execute (char *command);
@@ -41,12 +48,17 @@ extern int clear_before_exec;
 extern int confirm_delete;
 extern int mou_auto_repeat;
 extern char *other_dir;
+extern int auto_mount;
 
 /* Usefull macros to avoid too much typing */
 #define cpanel current_panel
 #define opanel other_panel
 
+typedef void (*key_callback)(int);
+
+/* The keymaps are of this type */
 typedef struct {
     int   key_code;
-    void  (*fn)(int);
+    key_callback fn;
 } key_map;
+
