@@ -19,6 +19,7 @@
 #include <stdarg.h>
 #include <sys/types.h>
 #include <ctype.h>
+#include <malloc.h>
 #include "menu.h"
 #include "dialog.h"
 #include "global.h"
@@ -27,8 +28,9 @@
 #include "main.h"
 #include "input.h"
 #include "mouse.h"
+#include "win.h"
 
-static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/menu.c,v 1.10 1994/09/01 23:56:27 miguel Exp $";
+static char rcsid [] = "$Id: menu.c,v 1.14 1994/10/13 04:27:52 miguel Exp $";
 
 extern int is_right;
 static int event_set;		/* Used to pass information from the */
@@ -61,7 +63,6 @@ static void paint_idx (Menu *menu, WINDOW *w, int idx, int sel, int len)
     wprintw (w, " %-*s ", len, menu->entries [idx].text);
 }
 
-#ifdef use_mouse
 int menu_handler (Gpm_Event *event, Menu *menu)
 {
     int pos;
@@ -87,7 +88,6 @@ int menu_handler (Gpm_Event *event, Menu *menu)
     }
     return MOU_NORMAL;
 }
-#endif
 
 int run_menu (Menu *menu, int y, int x)
 {
@@ -123,7 +123,7 @@ int run_menu (Menu *menu, int y, int x)
     push_event (x+2, y+2, x+max_entry_len+2, y+menu->count+2,
 		(mouse_h) menu_handler, menu);
 
-    while ((c = mi_getch ()) != '\e'){
+    while ((c = mi_getch ()) != ESC_CHAR && c != KEY_F(10)){
 	switch (c){
 	case KEY_UP:
 	case XCTRL('p'):		/* C-p */
@@ -203,7 +203,7 @@ int get_motion (Menu *Menus [], int top, int *abort)
     was_enter = 0;
     *abort = 0;
     
-    while ((c = mi_getch ()) != '\e'){
+    while ((c = mi_getch ()) != ESC_CHAR){
 	switch (c){
 	case '\n':
 	    return 0;
@@ -252,14 +252,24 @@ static void paint_bar (WINDOW *bar, int pos, int color, char *name)
 /* Ugly hack */
 int current = 0;
 
-#ifdef use_mouse
 int top_menu_handle (Gpm_Event *event, void *x)
 {
+    int is_bar = ((int) x) > 99;
+    int selection = (int) x;
+
+    selection = selection > 99 ? selection - 100 : selection;
+    
     if (event->type & (GPM_DOWN|GPM_DRAG)){
-	event_set = (int) x;
+	event_set = selection;
 	if (event_set != current)
 	    return MOU_ENDLOOP;
 	return MOU_NORMAL;
+    } else if ((event->type & GPM_UP)){
+	event_set = selection;
+
+	/* We just want to send a MOU_ENDLOOP to the pull down menus */
+	if (is_bar)
+	    return MOU_ENDLOOP;
     }
     return MOU_NORMAL;
 }
@@ -281,7 +291,6 @@ int quit_all_event (Gpm_Event *event, void *x)
     }
     return MOU_NORMAL;
 }
-#endif
 
 /* run_bar.
    WINDOW *parent, the parent window.
@@ -303,6 +312,7 @@ int run_bar (WINDOW *parent, int x, int y, int width, int space, int items,
 {
     int i;
     int abort;
+    int old_selection;
     WINDOW *bar;
     int is_bar;
     int fill  = (flags & BAR_FILL);
@@ -328,13 +338,17 @@ int run_bar (WINDOW *parent, int x, int y, int width, int space, int items,
     for (i = 0; i < items; i++){
 	wmove (bar, 0, i*space+2);
 	wprintw (bar, "%s", Menus [i]->name);
+
+	/* I use the is_bar * 100 to inform top_menu_handle the kind of */
+	/* menu it's running */
 	push_event (i*space+2+x, y+1,i*space+2+x+strlen (Menus [i]->name), y+1,
-		    top_menu_handle, (void *) i);
+		    top_menu_handle, (void *) (i + (is_bar * 100)));
     }
     do {
 	paint_bar (bar, current*space+2, color_sel, Menus [current]->name);
 	touchwin (bar);
 	wrefresh (bar);
+	old_selection = current;
 	is_right = current;
 	event_set = current;
 	if (is_bar)
@@ -343,7 +357,7 @@ int run_bar (WINDOW *parent, int x, int y, int width, int space, int items,
 	    i = run_menu (Menus [current], 1, current*space);
 	}
 	paint_bar (bar, current*space+2, color_uns, Menus [current]->name);
-
+	
 	if (i == QUIT_MENU){
 	    /* it means that the event was handled by a mouse handler */
 	    /* if event_set is == -1 then set i = 0 to exit the loop */
