@@ -13,9 +13,11 @@ typedef struct {
 } menu_entry;
 
 typedef struct {
-    char  *name;
-    int   count;
-    int   max_entry_len;
+    char   *name;
+    int    count;
+    int    max_entry_len;
+    int    selected;
+    WINDOW *win;
     menu_entry *entries;
 } Menu;
 
