@@ -10,7 +10,7 @@ typedef struct {
     char     cwd [MAXPATHLEN];	/* Current Working Directory */
     char     lwd [MAXPATHLEN];	/* Last Working Directory */
     int      count;		/* Number of files in dir structure */
-    int      lines;		/* Panel Lines */
+    int      lines;		/* Available panel lines for display */
     int      cols;		/* Panel columns */
     int      marked;		/* Count of marked file */
     long int total;		/* Bytes in marked files */
@@ -19,6 +19,7 @@ typedef struct {
     int      is_status;		/* Is the panel a status display? */
     int      reverse;		/* Show listing in reverse? */
     sortfn   *sort_type;	/* Sort type */
+    char     *filter;		/* File name filter */
     WINDOW   *win_file;		/* Where the files are listed */
     WINDOW   *win_mini;		/* The mini-info */
     
@@ -40,6 +41,8 @@ extern file_entry *selection;
 
 extern int fast_reload;
 
+extern int extra_info;
+
 /*#define ITEMS(p) ((p)->view_type == view_brief ? (p)->lines *2 : (p)->lines)
 */
 /* The return value of panel_reload */
@@ -63,3 +66,4 @@ void show_dir (Panel *panel);
 void brief_frame (Panel *panel);
 void full_frame (Panel *panel);
 void long_frame (Panel *panel);
+extern void paint_info_panel (Panel *);
