@@ -18,18 +18,31 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef HAVE_UNISTD_H
+#include <sys/types.h>
 #include <unistd.h>
+#endif
+#ifdef HAVE_LIBGEN_H
+#include <libgen.h>		/* For regcmp and regex */
+#endif
 #include <signal.h>		/* my_system */
 #include <limits.h>		/* INT_MAX */
+#include <sys/time.h>		/* select: timeout */
 #include <sys/param.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdarg.h>
 #include <sys/wait.h>		/* my_system */
 #include <errno.h>		/* my_system */
 #include <time.h>
 #include <pwd.h>
 #include <grp.h>
 #include <string.h>
+#include <ctype.h>
+/* From dialog.h (not wanting to include it as
+   it requires including a lot of other files, too) */
+int message (int error, char *header, char *text, ...);
+
 #if defined(linux) || defined(gnu_regex)
 #ifdef use_local_rx
 #include "rx.h"
@@ -39,10 +52,12 @@
 #endif
 #include "util.h"
 #include "global.h"
+#include "gc.h"
+#include "profile.h"
 
-static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/util.c,v 1.10 1994/09/08 19:28:30 miguel Exp $";
+static char rcsid [] = "$Id: util.c,v 1.13 1994/10/13 04:28:02 miguel Exp $";
 
-static char app_text [] = "MouseLess-Commander";
+static char app_text [] = "Midnight-Commander";
 
 int easy_patterns = 1;
 int align_extensions = 1;
@@ -204,11 +219,13 @@ char *strip_home(char *dir)
 	return dir;
 }
 
+#define MI_RE_COMP re_comp
+#define MI_RE_EXEC re_exec
 /* 1 if string matches
    0 if string doesn't match
    -1 if error in pattern */
 
-int regexp_match (char *pattern, char *string)
+int regexp_match (char *pattern, char *string, int match_type)
 {
     char *s, *d;
     char new_pattern [100];
@@ -220,7 +237,10 @@ int regexp_match (char *pattern, char *string)
 
     x = pattern;
     if (easy_patterns){
-	for (s = pattern, d = new_pattern; *s; s++, d++){
+	d = new_pattern;
+	if (match_type == match_file)
+	    *d++ = '^';
+	for (s = pattern; *s; s++, d++){
 	    switch (*s){
 	    case '*':
 		*d++ = '.';
@@ -238,6 +258,8 @@ int regexp_match (char *pattern, char *string)
 		*d = *s;
 	    }
 	}
+	if (match_type == match_file)
+	    *d++ = '$';
 	*d = 0;
 	pattern = new_pattern;
     }
@@ -248,11 +270,26 @@ int regexp_match (char *pattern, char *string)
     regfree (&r);
     return rval;
 #else
+#ifdef HAVE_RE_EXEC
     if (re_comp (pattern)){
 	return -1;
     }
     return re_exec (string);
-#endif    
+#else /* no re_exec */
+#ifdef HAVE_LIBGEN_H
+    s = regcmp (pattern, (char *) 0);
+    if (!s){
+	return -1;
+    }
+    d = regex (s, string);
+    free (s);
+    return d != 0;
+#else
+    /* No match, system without regex or re_exec */
+    return 0;
+#endif /* HAVE_LIBGEN_H */
+#endif /* HAVE_RE_EXEC */
+#endif /* defined(linux) */
 }
 
 char *extension (char *filename)
@@ -493,6 +530,87 @@ char *get_full_name (char *dir, char *file)
     return d;
 }
 
+/* Pipes are guaranteed to be able to hold at least 4096 bytes */
+/* More than that would be unportable */
+#define MAX_PIPE_SIZE 4096
+
+static int error_pipe[2];	/* File descriptors of error pipe */
+static int old_error;		/* File descriptor of old standard error */
+
+/* Creates a pipe to hold standard error for a later analysis. */
+/* The pipe can hold 4096 bytes. Make sure no more is written */
+/* or a deadlock might occur. */
+void open_error_pipe ()
+{
+    if (pipe (error_pipe) < 0){
+	message (0, " Warning ", " Pipe failed ");
+    }
+    old_error = dup (2);
+    if(old_error < 0 || close(2) || dup (error_pipe[1]) != 2){
+	message (0, " Warning ", " Dup failed ");
+	close (error_pipe[0]);
+	close (error_pipe[1]);
+    }
+    close (error_pipe[1]);
+}
+
+/* Checks for messages in the error pipe
+ * and displays an warning box if needed
+ */
+void check_error_pipe ()
+{
+    char error[MAX_PIPE_SIZE];
+    int len = 0;
+    if (old_error >= 0){
+	while (len < MAX_PIPE_SIZE)
+	{
+            struct fd_set select_set;
+            struct timeval timeout;
+            FD_ZERO (&select_set);
+            FD_SET (error_pipe[0], &select_set);
+            timeout.tv_sec = 0;
+            timeout.tv_usec = 0;
+            select (FD_SETSIZE, &select_set, 0, 0, &timeout);
+            if (!FD_ISSET (0, &select_set)) break;
+	    read (error_pipe[0], error + len, 1);
+	    len ++;
+	}
+	error[len] = 0;
+	close (error_pipe[0]);
+    }
+    if (len > 0)
+        message (0, " Warning ", error);
+}
+
+/* Checks for messages in the error pipe,
+ * closes the pipe and displays an error box if needed
+ */
+void close_error_pipe (int error, char *text)
+{
+    char *title;
+    char msg[MAX_PIPE_SIZE];
+    int len = 0;
+
+    if (error)
+	title = " Error ";
+    else
+	title = " Warning ";
+    if (old_error >= 0){
+	close (2);
+	dup (old_error);
+	close (old_error);
+	len = read (error_pipe[0], msg, MAX_PIPE_SIZE);
+	msg[len] = 0;
+	close (error_pipe[0]);
+    }
+    if (text == NULL){
+	if (len == 0) return;
+	message (error, title, msg);
+    } else {
+	message (error, title, " %s \n %s ", text, msg);
+    }
+}
+
 static struct sigaction startup_handler;
 
 int my_system (char *shell, char *command)
@@ -548,10 +666,6 @@ void save_stop_handler ()
     sigaction (SIGTSTP, NULL, &startup_handler);
 }
 
-#if defined(ultrix) || defined(linux)
-#define HAVE_STRERROR
-#endif
-
 char *unix_error_string (int error_num)
 {
     static char buffer [256];
@@ -570,3 +684,78 @@ char *unix_error_string (int error_num)
     sprintf (buffer, "%s (%d)", error_msg, error_num);
     return buffer;
 }
+
+char *tilde_expand (char *directory, int alloc_type)
+{
+    struct passwd *passwd;
+    char *p;
+    char *name;
+    int  len;
+    
+    if (*directory != '~')
+	return directory;
+
+    directory++;
+    
+    p = strchr (directory, '/');
+    
+    /* d = "~" or d = "~/" */
+    if (!(*directory) || (*directory == '/')){
+	passwd = getpwuid (geteuid ());
+	p = (*directory == '/') ? directory+1 : "";
+    } else {
+	if (!p){
+	    p = "";
+	    passwd = getpwnam (directory);
+	} else {
+	    name = xmalloc (p - directory + 1, "tilde_expand");
+	    strncpy (name, directory, p - directory);
+	    name [p - directory] = 0;
+	    passwd = getpwnam (name);
+	    free (name);
+	}
+    }
+
+    /* If we can't figure the user name, return home_dir */
+    if (!passwd)
+	return home_dir;
+
+    len = strlen (passwd->pw_dir) + strlen (p) + 2;
+    if (alloc_type == use_gc)
+	directory = gc_alloc (len);
+    else
+	directory = xmalloc (len, "tilde_expand");
+    strcpy (directory, passwd->pw_dir);
+    strcat (directory, "/");
+    strcat (directory, p);
+    return directory;
+}
+
+char *copy_strings (char *first,...)
+{
+    va_list ap;
+    int len;
+    char *data, *result;
+
+    if (!first)
+	return 0;
+    
+    len = strlen (first);
+    va_start (ap, first);
+
+    while ((data = va_arg (ap, char *))!=0)
+	len += strlen (data);
+
+    len++;
+
+    result = gc_alloc (len);
+    va_end (ap);
+    va_start (ap, first);
+    strcpy (result, first);
+    while ((data = va_arg (ap, char *)) != 0)
+	strcat (result, data);
+    va_end (ap);
+
+    return result;
+}
+	
