@@ -1,3 +1,5 @@
+/* String managing functions */
+
 int msglen (char *text, int *lines);
 char *trim (char *s, char *d, int len);
 char *name_trunc (char *txt, int trunc_len);
@@ -9,11 +11,13 @@ char *strip_home(char *dir);
 char *extension (char *);
 char *split_extension (char *, int pad);
 char *get_full_name (char *dir, char *file);
+char *copy_strings (char *first, ...);
+char *unix_error_string (int error_num);
 
-extern int easy_patterns;
 extern int align_extensions;
 void *xmalloc (int, char *);
 
+/* Profile managing functions */
 int set_int (char *, char *, int);
 int get_int (char *, char *, int);
 
@@ -24,8 +28,11 @@ char *load_file (char *filename);
 #define S_ISSOCK(x) (((x) & S_IFSOCK) == S_IFSOCK)
 #endif
 
+/* uid/gid managing */
+void init_uid_gid_cache ();
 char *get_group (int);
 char *get_owner (int);
+
 char *file_date (time_t);
 char *file_date_pck (time_t);
 int exist_file (char *name);
@@ -33,8 +40,23 @@ int exist_file (char *name);
 /* Returns a copy of *s until a \n is found and is below top */
 char *extract_line (char *s, char *top);
 char *icase_search (char *text, char *data);
-int regexp_match (char *pattern, char *string);
+
+/* Matching */
+enum { match_file, match_normal };
+extern int easy_patterns;
+int regexp_match (char *pattern, char *string, int match_type);
+
+/* Error pipes */
+void open_error_pipe ();
+void check_error_pipe ();
+void close_error_pipe (int error, char *text);
 
 int my_system (char *shell, char *command);
 void save_stop_handler ();
-void init_uid_gid_cache ();
+
+/* Tilde expansion */
+enum { use_gc, no_gc };
+char *tilde_expand (char *, int alloc_type);
+
+
+
